package edu.stanford.nlp.ie.machinereading.domains.bionlp;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.util.Collection;
import java.util.List;
import java.util.Properties;

import edu.stanford.nlp.ie.machinereading.GenericDataSetReader;
import edu.stanford.nlp.ie.machinereading.msteventextractor.DataSet;
import edu.stanford.nlp.ie.machinereading.msteventextractor.EpigeneticsDataSet;
import edu.stanford.nlp.ie.machinereading.msteventextractor.GENIA11DataSet;
import edu.stanford.nlp.ie.machinereading.msteventextractor.InfectiousDiseasesDataSet;
import edu.stanford.nlp.io.IOUtils;
import edu.stanford.nlp.ling.CoreLabel;
import edu.stanford.nlp.util.StringUtils;

/**
 * Standalone program to run our BioNLP tokenizer and save its output
 */
public class RunBioNLPTokenizer extends GenericDataSetReader {

  public static void main(String[] args) throws IOException {
    Properties props = StringUtils.argsToProperties(args);
    String basePath = props.getProperty("base.directory", "/u/nlp/data/bioNLP/2011/originals/");
    
    DataSet dataset = new GENIA11DataSet();
    dataset.getFilesystemInformation().setTokenizer("stanford");
    runTokenizerForDirectory(dataset, basePath + "genia/training");
    runTokenizerForDirectory(dataset, basePath + "genia/development");
    runTokenizerForDirectory(dataset, basePath + "genia/testing");
    
    dataset = new EpigeneticsDataSet();
    dataset.getFilesystemInformation().setTokenizer("stanford");
    runTokenizerForDirectory(dataset, basePath + "epi/training");
    runTokenizerForDirectory(dataset, basePath + "epi/development");
    runTokenizerForDirectory(dataset, basePath + "epi/testing");
    
    dataset = new InfectiousDiseasesDataSet();
    dataset.getFilesystemInformation().setTokenizer("stanford");
    runTokenizerForDirectory(dataset, basePath + "infect/training");
    runTokenizerForDirectory(dataset, basePath + "infect/development");
    runTokenizerForDirectory(dataset, basePath + "infect/testing");
  }

  private static void runTokenizerForDirectory(DataSet dataset, String path) throws IOException {
    System.out.println("Input directory: " + path);
    BioNLPFormatReader reader = new BioNLPFormatReader();    
    for (File rawFile : reader.getRawFiles(path)) {
      System.out.println("Input filename: " + rawFile.getName());
      String rawText = IOUtils.slurpFile(rawFile);

      String docId = rawFile.getName().replace("." + BioNLPFormatReader.TEXT_EXTENSION, "");
      String parentPath = rawFile.getParent();
      
      runTokenizer(dataset.getFilesystemInformation().getTokenizedFilename(parentPath, docId), rawText);
    }
  }

  private static void runTokenizer(String tokenizedFilename, String text) {
    System.out.println("Tokenized filename: " + tokenizedFilename);
    Collection<String> sentences = BioNLPFormatReader.splitSentences(text);

    PrintStream os = null;
    try {
      os = new PrintStream(new FileOutputStream(tokenizedFilename));
    } catch (IOException e) {
      System.err.println("ERROR: cannot save online tokenization to " + tokenizedFilename);
      e.printStackTrace();
      System.exit(1);
    }

    for (String sentence : sentences) {
      BioNLPFormatReader.BioNLPTokenizer tokenizer = new BioNLPFormatReader.BioNLPTokenizer(sentence);
      List<CoreLabel> tokens = tokenizer.tokenize();
      for (CoreLabel l : tokens) {
        os.print(l.word() + " ");
      }
      os.println();
    }
    os.close();
  }
}