package edu.stanford.nlp.ie.machinereading.domains.nfl;

import java.io.File;
import java.io.IOException;
import java.util.Properties;
import java.util.logging.Logger;

import edu.stanford.nlp.io.IOUtils;
import edu.stanford.nlp.pipeline.Annotation;
import edu.stanford.nlp.pipeline.StanfordCoreNLP;

/**
 * Sets up a pipeline and parses NFLGameRecaps made by NFLGameRecapCrawler
 * Annotates all files in the input directory.
 *
 * @author David McClosky
 */
public class NFLGameRecapParser {

  private Logger logger = null;
  private StanfordCoreNLP pipeline;

  public static void main(String[] args) throws Exception {
    if (args == null || args.length != 2) {
      System.out.println("Usage: NFLGameRecapParser inputDir outputDir");
      return;
    }
    String inputDir = args[0];
    String outputDir = args[1];
    NFLGameRecapParser parser = new NFLGameRecapParser();
    for (File inputFile : new File(inputDir).listFiles()) {
      
      // replace extension with ".ser"
      String outputFilename = inputFile.getName();
      int lastDot = outputFilename.lastIndexOf('.');
      // for paths like "./zzz", lastDot will be 0
      if (lastDot > 0) {
        outputFilename = outputFilename.substring(0, lastDot);
      }
      outputFilename += ".ser.gz";
      
      File outputFile = new File(outputDir, outputFilename);
      parser.parseAndSerialize(inputFile, outputFile);
    }
  }

	private void parseAndSerialize(File inputFile, File outputFile) throws IOException {
	  logger.info("Parsing " + inputFile.getCanonicalPath());
    
	  Annotation doc = new Annotation(IOUtils.slurpFile(inputFile));
	  pipeline.annotate(doc);
	  
	  logger.info("Serializing output to " + outputFile.getCanonicalPath());
	  IOUtils.writeObjectToFile(doc, outputFile);
  }
	
  public NFLGameRecapParser() {
    logger = Logger.getLogger(NFLReader.class.getName());
    Properties props = new Properties();
    props.setProperty("annotators", "cleanxml,ssplit,tokenize,pos,lemma,ner,parse,nfl");
    props.setProperty("clean.xmltags", "a|p");
    props.setProperty("clean.sentenceendingtags", "p");
    pipeline = new StanfordCoreNLP(props);
	}
}
