/* Copyright (C) 2002 Univ. of Massachusetts Amherst, Computer Science Dept.
   This file is part of "MALLET" (MAchine Learning for LanguagE Toolkit).
   http://www.cs.umass.edu/~mccallum/mallet
   This software is provided under the terms of the Common Public License,
   version 1.0, as published by http://www.opensource.org.  For further
   information, see the file `LICENSE' included with this distribution. */


package edu.stanford.nlp.ie.machinereading.msteventextractor;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;

/** 
 @author Andrew McCallum <a href="mailto:mccallum@cs.umass.edu">mccallum@cs.umass.edu</a>
 */
public class Alphabet implements Serializable {
  private gnu.trove.TObjectIntHashMap<Object> map;
  
  int numEntries;
  boolean growthStopped = false;

  public Alphabet(int capacity) {
    this.map = new gnu.trove.TObjectIntHashMap<Object>(capacity);
    numEntries = 0;
  }

  public Alphabet() {
    this(10000);
  }

  /** Return -1 if entry isn't present and growth is stopped.  Otherwise adds the entry. */
  public int lookupAndAddIndex(Object entry) {
    if (entry == null) {
      throw new IllegalArgumentException(
          "Can't lookup \"null\" in an Alphabet.");
    }

    if (map.containsKey(entry)) {
      return map.get(entry);
    } else {
      if (growthStopped)
        return -1;
      int ret = numEntries;
      map.put(entry, ret);
      numEntries++;
      return ret;
    }
  }
  
  /** Returns -1 if the entry isn't present always -- won't add it even if growth is not stopped. */
  public int lookupIndex(Object entry) {
    if (entry == null) {
      throw new IllegalArgumentException(
          "Can't lookup \"null\" in an Alphabet.");
    }

    if (map.containsKey(entry)) {
      return map.get(entry);
    } else {
      return -1;
    }
  }

  public Object[] toArray() {
    return map.keys();
  }

  public boolean contains(Object entry) {
    return map.containsKey(entry);
  }

  public int size() {
    return numEntries;
  }

  public void stopGrowth() {
    growthStopped = true;
    map.compact();
  }

  public void allowGrowth() {
    growthStopped = false;
  }

  public boolean growthStopped() {
    return growthStopped;
  }

  // Serialization

  private static final long serialVersionUID = 1;
  private static final int CURRENT_SERIAL_VERSION = 0;

  private void writeObject(ObjectOutputStream out) throws IOException {
    out.writeInt(CURRENT_SERIAL_VERSION);
    out.writeInt(numEntries);
    out.writeObject(map);
    out.writeBoolean(growthStopped);
  }

  @SuppressWarnings("unchecked")
  private void readObject(ObjectInputStream in) throws IOException,
      ClassNotFoundException {
    @SuppressWarnings("unused")
    int version = in.readInt();
    numEntries = in.readInt();
    map = (gnu.trove.TObjectIntHashMap<Object>) in.readObject();
    growthStopped = in.readBoolean();
  }
}
