package edu.stanford.nlp.ie.machinereading.msteventextractor;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import edu.stanford.nlp.util.StringUtils;


public class InfectiousDiseasesDataSet extends DataSet {
  
  public InfectiousDiseasesDataSet() {
    this("infect");
  }
  
  public InfectiousDiseasesDataSet(String shortName) {
    super();
    
    setFilesystemInformation(new BioNLP11FilesystemInformation(shortName, "default", "mccc"));
    
    entityTypes = new HashSet<String>(StringUtils.split("Protein Two-component-system Chemical Organism Regulon-operon Participant Organism/Protein Organism/Two-component-system Protein/Regulon-operon"));
    eventTypes = new HashSet<String>(StringUtils.split("Gene_expression Transcription Protein_catabolism Localization Phosphorylation Binding Regulation Positive_regulation Negative_regulation Process"));
    // technically, Process is not a single theme event since it shouldn't have any Themes, but this check is really just there
    // to make sure that it doesn't have *multiple* themes
    singleThemeEvents = new HashSet<String>(eventTypes);
    singleThemeEvents.remove("Binding");
    zeroArgEvents = new HashSet<String>(StringUtils.split("Process"));
    eventsWithOnlyThemes = new HashSet<String>(StringUtils.split("Gene_expression Transcription Protein_catabolism Localization Phosphorylation Binding"));
    eventsRequiringTheme = new HashSet<String>(eventTypes);
    eventsRequiringTheme.remove("Process");
    splitBindingEvents = true; // not sure if we want to do this for this dataset
    discardEmbeddedEntities = true;
  }

  public void addDomainSpecificFeatures(DependencyPipe pipe, DependencyInstance instance, int headIndex, int childIndex, FeatureVector fv) {
    // none for now
  }

  public void addDomainSpecificLabeledFeatures(DependencyPipe pipe, DependencyInstance instance, int index, String labelName,
      boolean attachingRight, boolean childFeatures, FeatureVector fv) {
    // none for now
  }

  public Map<String, Set<String>> buildTypeGeneralizations() {
    HashMap<String,Set<String>> typeGeneralizations = new HashMap<String, Set<String>>();
    Set<String> simpleEventTypes = new HashSet<String>(StringUtils.split("Gene_expression Transcription Protein_catabolism Localization Phosphorylation Binding"));
    Set<String> complexEventTypes = new HashSet<String>(StringUtils.split("Regulation Positive_regulation Negative_regulation"));
    typeGeneralizations.put("simple", simpleEventTypes);
    typeGeneralizations.put("complex", complexEventTypes);
    
    // all event types
    Set<String> allEventTypes = new HashSet<String>(simpleEventTypes);
    allEventTypes.addAll(complexEventTypes);
    allEventTypes.add("Process");
    typeGeneralizations.put("events", allEventTypes);

    // degenerate generalization for participants
    Set<String> participantTypes = new HashSet<String>(StringUtils.split("Participant"));
    typeGeneralizations.put("participant", participantTypes);
    
    // type of args for most slots
    Set<String> coreEntityTypes = new HashSet<String>(StringUtils.split("Protein Two-component-system Chemical Organism Regulon-operon"));
    typeGeneralizations.put("core-entity", coreEntityTypes);
    
    // type of args for Gene_expression and Transcription
    Set<String> proteinOrRegTypes = new HashSet<String>(StringUtils.split("Protein Regulon-operon"));
    typeGeneralizations.put("protein-or-reg", proteinOrRegTypes);
    
    return typeGeneralizations;
  }

  public boolean includeRole(String role) {
    String roleLower = role.toLowerCase();
    // just the Task 1 roles for now
    return (roleLower.startsWith("theme") || roleLower.startsWith("cause") || roleLower.startsWith("participant"));
  }
  
}
